/*
Tecnisa - Tecnologia em Informtica

Nome: APURAC_TAXAFIXA

Descrio:
   Procedimento para apurao de faturamento de clientes em filiais
   que utilizam o tipo de contrato Taxa Fixa.

Criao:
22/01/2002 - Disnei H. de Moura

@C:\win_V1.53\SQL\ORACLE\6206_FAISTP_ApuTaxaFixa.sql;

--   raise_application_error(-20501,TO_CHAR(V_VRBRUTITEM)||' - '||TO_CHAR(V_VRBRUTITEM), True);
*/


CREATE OR REPLACE PROCEDURE APURAC_TAXAFIXA(
   P_CDFILIAPUR  IN FILIAL.CDFILIAL%TYPE,
   P_CDFILIPROC  IN FILIAL.CDFILIAL%TYPE,
   P_CDCLIENTE   IN CLIENTE.CDCLIENTE%TYPE,
   P_DATAINI     IN DATE,
   P_DATAFIM     IN DATE,
   P_NRAPURACFAT IN NOTAFISCAL.NRAPURACFAT%TYPE,
   P_NRLANCTONF  IN NOTAFISCAL.NRLANCTONF%TYPE,
   P_IDNFCOMPRA  IN VARCHAR) AS

   V_CDEMPRATAC  EMPRESA.CDEMPRESA%TYPE;
   V_CDINSCATAC  INSCRESTAD.CDINSCESTA%TYPE;
   V_CDEMPROPER  EMPRESA.CDEMPRESA%TYPE;
   V_CDSERIE     SERIENF.CDSERIE%TYPE;
   V_CDIMPOSTO   IMPOSTO.CDIMPOSTO%TYPE;

   V_CDFILIAPUR  CTRLAPURFAT.CDFILIAPUR%TYPE;

   V_NRLANCTONF  NOTAFISCAL.NRLANCTONF%TYPE;
   V_VRNOTAFISC  NOTAFISCAL.VRNOTAFISC%TYPE;

   V_VRDOTACAO  ITEMPRODUTO.VRTOTAITEM%TYPE;


   V_NRLANCESTQ  LANCTOESTOQ.NRLANCESTQ%TYPE;

   V_CDPRGENEREV PRODUTO.CDPRODUTO%TYPE;
   V_NMPRGENEREV PRODUTO.NMPRODUTO%TYPE;

   V_DSAJUCTREC  AJCTRECEBER.DSAJUSTEREC%TYPE;

   V_ENCARGO     ENCARGO.CDENCARGO%TYPE;

   V_NVPRGENEREV    VARCHAR2(20);

   V_DSPRPRESERV VARCHAR2(40);
   V_VERIGRUPDES VARCHAR2(02);
   V_IDTAXAADM   VARCHAR2(01);
   V_IDCALCCOMPR VARCHAR2(01);
   V_IDRECUIMPOS VARCHAR2(01);
   V_DESCDOTACAO VARCHAR2(01);
   V_CDAJUSTPADR VARCHAR2(02);
   V_CDENCREDITO VARCHAR2(02);
   V_CDENCDEBITO VARCHAR2(02);
   V_IDGERACORPO VARCHAR2(01);


   V_VRBRUTITEM  NUMBER;
   v_VRIMPOSTOIT NUMBER;
   V_VRBRUTDESP  NUMBER;
   V_VRTAXAADM   NUMBER;
   V_VRPCTXADM   NUMBER;
   V_VRLIQUFATU  NUMBER;
   V_PRXDESPESA  NUMBER;
   V_VRIMPODEBI  NUMBER;
   V_VRDEBITO    NUMBER;
   V_VRPERCDESP  NUMBER;

   V_CDNIVELPROD NUMBER;

   TESTE         NUMBER;
   V_PRXMRDP     NUMBER;

   /*Cursor auxiliar para deletar as notas fiscais*/
   CURSOR C_DELETANF IS SELECT CDFILIAL, NRLANCTONF
                        FROM NOTAFISCAL
                        WHERE CDFILIAL    = P_CDFILIAPUR AND
                              NRAPURACFAT = P_NRAPURACFAT;

   CURSOR C_DESPADMFATUR IS  SELECT L.CDFILIAL, L.CDDESPADMFAT, L.DTLANCDESFAT, L.VRLANCDESFAT,
                                    D.CDGRUPOFATUR, D.CDPRODUTO, D.DSDESPADMFAT
                             FROM LANCDESPFATU L,  DESPADMFATUR D
                             WHERE (L.CDFILIAL = P_CDFILIAPUR)                            AND
                                   (D.CDDESPADMFAT = L.CDDESPADMFAT)                      AND
                                   (L.DTLANCDESFAT BETWEEN P_DATAINI AND P_DATAFIM)
                             GROUP BY D.CDGRUPOFATUR, L.CDFILIAL, L.CDDESPADMFAT, L.DTLANCDESFAT, L.VRLANCDESFAT,
                                      D.CDPRODUTO, D.DSDESPADMFAT;


BEGIN

   V_VRNOTAFISC := 0;

   /* FILIAL QUE EST SENDO APURADA (P_FILIAL = FILIPROC)*/
   SELECT MAX(CDFILIAPUR)
   INTO V_CDFILIAPUR
   FROM CTRLAPURFAT
   WHERE CDFILIAL    = P_CDFILIPROC   AND
         NRAPURACFAT = P_NRAPURACFAT;

   SELECT TO_NUMBER(NVL(MAX(SUBSTR(PF.DSPARAMFATFI,31,6)),0)), /*Percentual de Compras/Revenda*/
          MAX(SUBSTR(PF.DSPARAMFATFI,39,2)),          /*Empresa atacadista*/
          MAX(SUBSTR(PF.DSPARAMFATFI,41,2)),          /*Empresa operacional*/
          MAX(SUBSTR(PF.DSPARAMFATFI,43,1)),          /*Taxa de Administracao*/
          MAX(SUBSTR(PG.DSPARAMFATGE,22,40)),         /*Descricao para Nota Fiscal*/
          RTRIM(MAX(SUBSTR(PG.DSPARAMFATGE,125,10))),  /*Codigo da Produto para Revenda*/
          RTRIM(MAX(SUBSTR(PG.DSPARAMFATGE,135,20))), /* Nvel dos produtos a serem gerados*/
          MAX(SUBSTR(PF.DSPARAMFATFI,49,1)),          /*Calculo Valor Total de Compras*/
          MAX(SUBSTR(PF.DSPARAMFATFI,37,1)),          /*Recuperao de Impostos*/
          MAX(SUBSTR(PF.DSPARAMFATFI,56,1)),          /*Desconto de Dotao*/
          MAX(SUBSTR(PF.DSPARAMFATFI,57,2)),          /*Ajuste Padrao*/
          MAX(SUBSTR(PF.DSPARAMFATFI,59,2)),          /*Encargo debito*/
          MAX(SUBSTR(PF.DSPARAMFATFI,61,2)),           /*Encargo Credito*/
          MAX(SUBSTR(PF.DSPARAMFATFI,63,5))         /*Percentual de Despesas*/
   INTO V_VRPCTXADM,   V_CDEMPRATAC,  V_CDEMPROPER,  V_IDTAXAADM,
        V_DSPRPRESERV, V_CDPRGENEREV, V_NVPRGENEREV, V_IDCALCCOMPR,
        V_IDRECUIMPOS, V_DESCDOTACAO, V_CDAJUSTPADR, V_CDENCREDITO,
        V_CDENCDEBITO, V_VRPERCDESP
   FROM PARAMFILIAL PF, PARAMGERAL PG
   WHERE PF.CDFILIAL = V_CDFILIAPUR;

   V_VRPCTXADM    := V_VRPCTXADM/1000;
   V_VRPERCDESP   := V_VRPERCDESP/1000;

   SELECT MAX(NMPRODUTO)
   INTO V_NMPRGENEREV
   FROM PRODUTO
   WHERE CDPRODUTO  =  V_CDPRGENEREV;

   SELECT SUM(VRNOTAFISC)
   INTO V_VRNOTAFISC
   FROM NOTAFISCAL N
   WHERE (N.CDFILIAL     = P_CDFILIAPUR)                  AND
         (N.CDTIPOOPER   = '12')                          AND
         (N.IDENTRSAIDOP = 'S')                           AND
         (N.DTENTRSAID BETWEEN P_DATAINI AND P_DATAFIM);

   IF V_VRNOTAFISC < 0 THEN
      V_VRNOTAFISC := V_VRNOTAFISC * (-1);
      V_ENCARGO    := V_CDENCDEBITO;
   ELSIF V_VRNOTAFISC > 0 THEN
      V_ENCARGO    :=  V_CDENCREDITO;
   END IF;

   SELECT LENGTH(MAX(CDPRODUTO))
   INTO V_CDNIVELPROD
   FROM PRODUTO
   WHERE CDNVPRODUTO = '1';

   SELECT MAX(SUBSTR(DSAJUCTREC,1,20))
   INTO V_DSAJUCTREC
   FROM TIPOAJRECEBER
   WHERE V_CDAJUSTPADR = CDAJUCTREC;

   IF P_IDNFCOMPRA = 'N' THEN
      UPDATE  NOTAFISCAL
      SET CDFILIAPFATU = P_CDFILIPROC,
          NRAPFATUTNF  = P_NRAPURACFAT
      WHERE (CDEMPRESA    = V_CDEMPRATAC) AND
            (CDFILIRECE   = V_CDFILIAPUR) AND
            (DTENTRSAID BETWEEN  P_DATAINI AND P_DATAFIM)    AND
            (IDENTRSAIDOP = 'E')          AND
            (IDSTATUS     <> 'C')         AND
            (NVL(IDSISTEMA,'DFE') IN ('DFE','EST')) AND
            ((NRAPFATUTNF IS NULL) OR (NRAPFATUTNF = P_NRAPURACFAT));
   END IF;

   SELECT MAX(CDSERIE)
   INTO V_CDSERIE
   FROM FILIAL
   WHERE CDFILIAL  = V_CDFILIAPUR AND
         CDEMPRESA = V_CDEMPRATAC;

   SELECT MAX(CDIMPOSTO)
   INTO V_CDIMPOSTO
   FROM IMPOSMRDP
   WHERE IDIMPOMRDP = 'P';

   V_NRLANCTONF := P_NRLANCTONF;

   V_IDGERACORPO := ' ';

   FOR C_NOTAFISCAL IN C_DELETANF LOOP
      DELETA_NF(C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.NRLANCTONF);
   END LOOP;

   FOR C_NFCOMPRA IN (
      SELECT NVL(SUM(IP.VRBRUTITEM),0)  AS VRBRUTITEM,
             NVL(SUM(II.VRIMPOSTOIT),0) AS VRIMPOSTOIT,
             NVL(SUM(DV.VRBRUTITEM),0)   AS VRBRUTDESP,
             NVL(SUM(DV.VRIMPOSTOIT),0)  AS VRDEBITO,
             PR.NMPRODUTO, PR.CDPRODUTO
      FROM  ITEMPRODUTO IP,  NOTAFISCAL N, ITEMPROIMPOS II,
           (SELECT CDPRODUTO, NMPRODUTO
            FROM PRODUTO
            WHERE CDNVPRODUTO = '1') PR,
           (SELECT I.VRBRUTITEM, T.VRIMPOSTOIT,
                   CDPRODUTO
            FROM  ITEMPRODUTO I, NOTAFISCAL N, ITEMPROIMPOS  T
            WHERE (CDEMPRESA    = V_CDEMPRATAC)         AND
                  (CDFILIEMIT   = V_CDFILIAPUR)         AND
                  (DTENTRSAID  <= P_DATAFIM)            AND
                  (CDTIPOOPER   IN ('06','01'))                 AND
                  (IDENTRSAIDOP = 'S')                  AND
                  (IDSTATUS IN ('E','A'))               AND
                  (N.CDFILIAL        = I.CDFILIAL)      AND
                  (N.NRLANCTONF      = I.NRLANCTONF)    AND
                  (T.CDFILIAL(+)     = I.CDFILIAL)      AND
                  (T.NRLANCTONF(+)   = I.NRLANCTONF)    AND
                  (T.NRSEQITPROD(+)  = I.NRSEQITPROD)   AND
                  (T.CDIMPOSTO(+)    = V_CDIMPOSTO)     AND
                  (T.NRITPROIMPOS(+) = '1')             AND
                  (INSTR(V_NVPRGENEREV,SUBSTR(I.CDPRODUTO,1,V_CDNIVELPROD)) > 0)) DV
      WHERE N.CDFILIAPFATU     = P_CDFILIPROC                              AND
            N.NRAPFATUTNF      = P_NRAPURACFAT                             AND
            IP.CDFILIAL        = N.CDFILIAL                                AND
            IP.NRLANCTONF      = N.NRLANCTONF                              AND
            SUBSTR(IP.CDSITUTRIBPR,2,2) NOT IN ('10','60')                 AND
            II.CDFILIAL(+)     = IP.CDFILIAL                               AND
            II.NRLANCTONF(+)   = IP.NRLANCTONF                             AND
            II.NRSEQITPROD(+)  = IP.NRSEQITPROD                            AND
            II.CDIMPOSTO(+)    = V_CDIMPOSTO                               AND
            II.NRITPROIMPOS(+) = '1'                                       AND
            DV.CDPRODUTO(+)      = IP.CDPRODUTO                            AND
            INSTR(V_NVPRGENEREV,SUBSTR(IP.CDPRODUTO,1,V_CDNIVELPROD)) > 0  AND
            INSTR(PR.CDPRODUTO,SUBSTR(IP.CDPRODUTO,1,V_CDNIVELPROD)) > 0
      GROUP BY  PR.NMPRODUTO, PR.CDPRODUTO

      UNION ALL

      SELECT NVL(SUM(IP.VRBRUTITEM),0)  AS VRBRUTITEM,
             NVL(SUM(II.VRIMPOSTOIT),0) AS VRIMPOSTOIT,
             NVL(SUM(DV.VRBRUTITEM),0)   AS VRBRUTDESP,
             NVL(SUM(DV.VRIMPOSTOIT),0)  AS VRDEBITO,
             PR.NMPRODUTO, PR.CDPRODUTO
      FROM  ITEMPRODUTO IP,  NOTAFISCAL N,
            ITEMPROIMPOS II, PRODUTO PR,
           (SELECT I.VRBRUTITEM,
                   T.VRIMPOSTOIT,
                   CDPRODUTO
            FROM  ITEMPRODUTO I, NOTAFISCAL N, ITEMPROIMPOS  T
            WHERE (CDEMPRESA    = V_CDEMPRATAC)         AND
                  (CDFILIEMIT   = V_CDFILIAPUR)         AND
                  (DTENTRSAID  <= P_DATAFIM)            AND
                  (CDTIPOOPER   = '06')                 AND
                  (IDENTRSAIDOP = 'S')                  AND
                  (IDSTATUS IN ('E','A'))               AND
                  (N.CDFILIAL        = I.CDFILIAL)      AND
                  (N.NRLANCTONF      = I.NRLANCTONF)    AND
                  (T.CDFILIAL(+)     = I.CDFILIAL)      AND
                  (T.NRLANCTONF(+)   = I.NRLANCTONF)    AND
                  (T.NRSEQITPROD(+)  = I.NRSEQITPROD)   AND
                  (T.CDIMPOSTO(+)    = V_CDIMPOSTO)     AND
                  (T.NRITPROIMPOS(+) = '1')             AND
                  (INSTR(V_NVPRGENEREV,SUBSTR(I.CDPRODUTO,1,V_CDNIVELPROD)) > 0)) DV
      WHERE N.CDFILIAPFATU     = P_CDFILIPROC                              AND
            N.NRAPFATUTNF      = P_NRAPURACFAT                             AND
            IP.CDFILIAL        = N.CDFILIAL                                AND
            IP.NRLANCTONF      = N.NRLANCTONF                              AND
            SUBSTR(IP.CDSITUTRIBPR,2,2) IN ('10','60')                     AND
            II.CDFILIAL(+)     = IP.CDFILIAL                               AND
            II.NRLANCTONF(+)   = IP.NRLANCTONF                             AND
            II.NRSEQITPROD(+)  = IP.NRSEQITPROD                            AND
            II.CDIMPOSTO(+)    = V_CDIMPOSTO                               AND
            II.NRITPROIMPOS(+) = '1'                                       AND
            IP.CDPRODUTO       = PR.CDPRODUTO                              AND
            DV.CDPRODUTO(+)      = IP.CDPRODUTO                            AND
            INSTR(V_NVPRGENEREV,SUBSTR(IP.CDPRODUTO,1,V_CDNIVELPROD)) > 0
      GROUP BY  PR.NMPRODUTO, PR.CDPRODUTO) LOOP

      IF V_IDGERACORPO <> 'X' THEN
         /*Incluso do Corpo da Nota */

         GERA_NOTAFISCAL_CORPO(P_CDFILIPROC, V_CDFILIAPUR, V_CDEMPRATAC, '01',
                               V_CDSERIE, P_CDCLIENTE, P_DATAFIM, 'FAI',
                               V_NRLANCTONF, V_NRLANCESTQ, 'N', P_NRAPURACFAT,
                               V_NRLANCTONF);

         IF (V_DESCDOTACAO = '1') AND (V_VRNOTAFISC <> 0) THEN
            GERA_NOTAFISCAL_ENCARGO(P_CDFILIPROC, V_NRLANCTONF, V_ENCARGO, V_VRNOTAFISC);
         END IF;
         V_IDGERACORPO := 'X';
      END IF;

      IF V_IDRECUIMPOS = 'S' THEN
        V_VRIMPODEBI := NVL(C_NFCOMPRA.VRIMPOSTOIT,0) - NVL(C_NFCOMPRA.VRDEBITO,0);
      ELSE
        V_VRIMPODEBI := 0;
      END IF;

      V_VRBRUTITEM  := (C_NFCOMPRA.VRBRUTITEM  -  C_NFCOMPRA.VRBRUTDESP) +  V_VRIMPODEBI;

      IF V_IDTAXAADM = 'D' THEN  /*Por Dentro*/
          V_VRTAXAADM := (V_VRBRUTITEM / (1 - (V_VRPCTXADM / 100))) - (V_VRBRUTITEM);
      ELSE                       /*Por Fora*/
          V_VRTAXAADM := (V_VRBRUTITEM * (V_VRPCTXADM / 100));
      END IF;

      IF V_IDCALCCOMPR = '1' THEN
         V_VRLIQUFATU := (V_VRBRUTITEM + NVL(V_VRTAXAADM,0));
      ELSE
         V_VRLIQUFATU :=  NVL(V_VRTAXAADM,0);
      END IF;

      /*Incluso dos itens da nota fiscal*/
      GERA_NOTAFISCAL_PRODUTO(P_CDFILIPROC, V_NRLANCTONF, V_CDPRGENEREV,
                              C_NFCOMPRA.CDPRODUTO, C_NFCOMPRA.NMPRODUTO, 1, V_VRLIQUFATU, 0,
                              NULL, NULL, NULL, NULL,
                              NULL, NULL);


   END LOOP;

   /*Incluso do itens da despesa administrativas lanadas no FAI*/


   FOR C_DESPESA IN C_DESPADMFATUR LOOP

      IF V_IDTAXAADM = 'D' THEN  /*Por Dentro*/
         V_VRTAXAADM := ((C_DESPESA.VRLANCDESFAT / (1 - (V_VRPCTXADM / 100)))+ (C_DESPESA.VRLANCDESFAT * (V_VRPERCDESP / 100))) - (C_DESPESA.VRLANCDESFAT);
      ELSE                       /*Por Fora*/
         V_VRTAXAADM := ((C_DESPESA.VRLANCDESFAT * (V_VRPCTXADM / 100)) + (C_DESPESA.VRLANCDESFAT * (V_VRPERCDESP / 100)));
      END IF;

      V_VRLIQUFATU := (C_DESPESA.VRLANCDESFAT + NVL(V_VRTAXAADM,0));

      IF V_IDGERACORPO <> 'X' THEN
            /*Incluso do Corpo da Nota */
           GERA_NOTAFISCAL_CORPO(P_CDFILIPROC, V_CDFILIAPUR, V_CDEMPRATAC, '01',
                                V_CDSERIE, P_CDCLIENTE, P_DATAFIM, 'FAI',
                                V_NRLANCTONF, V_NRLANCESTQ, 'N', P_NRAPURACFAT,
                                V_NRLANCTONF);
         V_IDGERACORPO := 'X';
      END IF;

      IF V_IDGERACORPO = 'X' THEN
         /*Incluso dos itens da nota fiscal*/
         GERA_NOTAFISCAL_PRODUTO(P_CDFILIPROC, V_NRLANCTONF, C_DESPESA.CDPRODUTO,
                                 C_DESPESA.CDDESPADMFAT, C_DESPESA.DSDESPADMFAT, 1, V_VRLIQUFATU, 0,
                                 NULL, NULL, NULL, NULL,
                                  NULL, NULL);
      /*Efetua o fechamento da nota, geracao de impostos*/
         GERA_NOTAFISCAL_FECHAMENTO(P_CDFILIPROC, V_NRLANCTONF);
      END IF;

      UPDATE LANCDESPFATU
      SET CDFILILANC = P_CDFILIPROC,
          NRLANCTONF = V_NRLANCTONF
      WHERE (CDFILIAL = P_CDFILIAPUR)                        AND
            (CDDESPADMFAT = C_DESPESA.CDDESPADMFAT)          AND
            (DTLANCDESFAT BETWEEN P_DATAINI AND P_DATAFIM);

   END LOOP;

   IF (V_DESCDOTACAO = '2') AND (V_VRNOTAFISC <> 0) THEN
      INSERT INTO AJCTRECEBER
      (CDFILIAL,    NRLANCTONF,  DTAJUSTEREC, CDAJUCTREC,
       DSAJUSTEREC, VRAJUSTEREC, NRSEQVENREC, NRPARCELAREC)
      (SELECT MAX(CDFILIAL),  MAX(NRLANCTONF), SYSDATE , V_CDAJUSTPADR,
           V_DSAJUCTREC,   ROUND(V_VRNOTAFISC,2), MAX(NRSEQVENREC), MAX(NRPARCELAREC)
        FROM CONTARECEBER
       WHERE (CDFILIAL    =  P_CDFILIPROC)  AND
             (NRLANCTONF  =  V_NRLANCTONF));

   END IF;

END;
/

